@@ -1,3 +1,3 @@
 from .syncobj import SyncObj, SyncObjException, SyncObjConf, replicated, replicated_sync,\
     FAIL_REASON, _COMMAND_TYPE, createJournal, HAS_CRYPTO, SERIALIZER_STATE, SyncObjConsumer, _RAFT_STATE
-from .syncobj_admin import Utility
+from .utility import TcpUtility
