@@ -34,6 +34,7 @@ class Transport(object):
         self._onNodeDisconnectedCallback = None
         self._onReadonlyNodeConnectedCallback = None
         self._onReadonlyNodeDisconnectedCallback = None
+        self._onUtilityMessageCallbacks = {}
 
     def setOnMessageReceivedCallback(self, callback):
         """
@@ -85,6 +86,21 @@ class Transport(object):
 
         self._onReadonlyNodeDisconnectedCallback = callback
 
+    def setOnUtilityMessageCallback(self, message, callback):
+        """
+        Set the callback for when an utility message is received, or disable callback by passing None
+
+        :param message: the utility message string (add, remove, set_version, and so on)
+        :type message: str
+        :param callback: callback
+        :type callback: function(message: list, callback: function) or None
+        """
+
+        if callback:
+            self._onUtilityMessageCallbacks[message] = callback
+        elif message in self._onUtilityMessageCallbacks:
+            del self._onUtilityMessageCallbacks[message]
+
     # Helper functions so you don't need to check for the callbacks manually in subclasses
     def _onMessageReceived(self, node, message):
         if self._onMessageReceivedCallback is not None:
@@ -243,11 +259,21 @@ class TCPTransport(Transport):
         """
 
         conf = self._syncObj.conf
-        bindAddr = conf.bindAddress or getattr(self._selfNode, 'address')
-        if not bindAddr:
+        bindAddr = conf.bindAddress
+        seflAddr = getattr(self._selfNode, 'address')
+        if bindAddr is not None:
+            host, port = bindAddr.rsplit(':', 1)
+        elif seflAddr is not None:
+            host, port = seflAddr.rsplit(':', 1)
+            if ':' in host:
+                host = '::'
+            else:
+                host = '0.0.0.0'
+        else:
             raise RuntimeError('Unable to determine bind address')
-        host, port = bindAddr.rsplit(':', 1)
-        host = globalDnsResolver().resolve(host)
+        
+        if host != '0.0.0.0':
+            host = globalDnsResolver().resolve(host)
         self._server = TcpServer(self._syncObj._poller, host, port, onNewConnection = self._onNewIncomingConnection,
                                  sendBufferSize = conf.sendBufferSize,
                                  recvBufferSize = conf.recvBufferSize,
@@ -318,29 +344,8 @@ class TCPTransport(Transport):
             return
 
         # Utility messages
-        if isinstance(message, list):
-            done = False
-            try:
-                if message[0] == 'status':
-                    conn.send(self._syncObj.getStatus())
-                    done = True
-                elif message[0] == 'add':
-                    self._syncObj.addNodeToCluster(message[1], callback = functools.partial(self._utilityCallback, conn = conn, cmd = 'ADD', arg = message[1]))
-                    done = True
-                elif message[0] == 'remove':
-                    if message[1] == self._selfNode.address:
-                        conn.send('FAIL REMOVE ' + message[1])
-                    else:
-                        self._syncObj.removeNodeFromCluster(message[1], callback = functools.partial(self._utilityCallback, conn = conn, cmd = 'REMOVE', arg = message[1]))
-                    done = True
-                elif message[0] == 'set_version':
-                    self._syncObj.setCodeVersion(message[1], callback = functools.partial(self._utilityCallback, conn = conn, cmd = 'SET_VERSION', arg = str(message[1])))
-                    done = True
-            except Exception as e:
-                conn.send(str(e))
-                done = True
-            if done:
-                return
+        if isinstance(message, list) and self._onUtilityMessage(conn, message):
+            return
 
         # At this point, message should be either a node ID (i.e. address) or 'readonly'
         node = self._nodeAddrToNode[message] if message in self._nodeAddrToNode else None
@@ -365,21 +370,31 @@ class TCPTransport(Transport):
         else:
             self._onReadonlyNodeConnected(node)
 
-    def _utilityCallback(self, res, err, conn, cmd, arg):
+    def _onUtilityMessage(self, conn, message):
+        command = message[0]
+        if command in self._onUtilityMessageCallbacks:
+            message[0] = command.upper()
+            callback = functools.partial(self._utilityCallback, conn = conn, args = message)
+            try:
+                self._onUtilityMessageCallbacks[command](message[1:], callback)
+            except Exception as e:
+                conn.send(str(e))
+            return True
+
+    def _utilityCallback(self, res, err, conn, args):
         """
         Callback for the utility messages
 
         :param res: result of the command
         :param err: error code (one of pysyncobj.config.FAIL_REASON)
         :param conn: utility connection
-        :param cmd: command
-        :param arg: command arguments
+        :param args: command with arguments
         """
 
-        cmdResult = 'FAIL'
-        if err == FAIL_REASON.SUCCESS:
-            cmdResult = 'SUCCESS'
-        conn.send(cmdResult + ' ' + cmd + ' ' + arg)
+        if not (err is None and res):
+            cmdResult = 'SUCCESS' if err == FAIL_REASON.SUCCESS else 'FAIL'
+            res = ' '.join(map(str, [cmdResult] + args))
+        conn.send(res)
 
     def _shouldConnect(self, node):
         """
@@ -417,6 +432,12 @@ class TCPTransport(Transport):
         for node in self._nodes:
             self._connectIfNecessarySingle(node)
 
+    def _sendSelfAddress(self, conn):
+        if self._selfIsReadonlyNode:
+            conn.send('readonly')
+        else:
+            conn.send(self._selfNode.address)
+
     def _onOutgoingConnected(self, conn):
         """
         Callback for when a new connection from this to another node is established. Handles encryption and informs the other node which node this is.
@@ -432,11 +453,8 @@ class TCPTransport(Transport):
             conn.recvRandKey = os.urandom(32)
             conn.send(conn.recvRandKey)
         else:
+            self._sendSelfAddress(conn)
             # The onMessageReceived callback is configured in addNode already.
-            if not self._selfIsReadonlyNode:
-                conn.send(self._selfNode.address)
-            else:
-                conn.send('readonly')
             self._onNodeConnected(self._connToNode(conn))
 
     def _onOutgoingMessageReceived(self, conn, message):
@@ -452,7 +470,7 @@ class TCPTransport(Transport):
 
         if not conn.sendRandKey:
             conn.sendRandKey = message
-            conn.send(self._selfNode.address)
+            self._sendSelfAddress(conn)
 
         node = self._connToNode(conn)
         conn.setOnMessageReceivedCallback(functools.partial(self._onMessageReceived, node))
